# frozen_string_literal: true

module Arcamark
  module CharType
  module_function

    def invalid?(char)
      char <= ' ' && !newline?(char) && !whitespace?(char)
    end

    def newline?(char)
      char == "\n"
    end

    def whitespace?(char)
      char == ' '
    end

    def operator?(char)
      %w[{ }].include? char
    end

    def funcall_start?(char)
      char == '@'
    end

    def funcall?(char)
      funcall_start?(char) ||
        char == '_' ||
        char >= 'a' && char <= 'z' ||
        char >= '0' && char <= '9'
    end

    def text?(char)
      !invalid?(char) &&
        !newline?(char) &&
        !whitespace?(char) &&
        !operator?(char) &&
        !funcall_start?(char)
    end
  end
end
