# frozen_string_literal: true

module Arcamark
  class Token
    TYPES = %i[empty_line whitespace text brace_open brace_close funcall].freeze

    attr_reader :type, :value, :index, :line, :column

    def initialize(type, value, index, line, column)
      self.type   = type
      self.value  = value
      self.index  = index
      self.line   = line
      self.column = column
    end

    def inspect
      "#<#{self.class}:#{index}:#{line}:#{column}:#{type}:#{inspect_value}>"
    end

    def inspect_value
      inner = value.inspect[1...-1]
      if inner.length > 20
        "\"#{inner[0...17]}\"..."
      else
        "\"#{inner}\""
      end
    end

  private

    def type=(type)
      raise 'Invalid type' unless TYPES.include? type

      @type = type
    end

    def value=(value)
      unless value.instance_of? String
        raise TypeError, "Expected #{String}, got #{value.class}"
      end

      @value = value
    end

    def index=(index)
      unless index.instance_of? Integer
        raise TypeError, "Expected #{Integer}, got #{index.class}"
      end
      raise 'Invalid index' unless index >= 0

      @index = index
    end

    def line=(line)
      unless line.instance_of? Integer
        raise TypeError, "Expected #{Integer}, got #{line.class}"
      end
      raise 'Invalid line' unless line.positive?

      @line = line
    end

    def column=(column)
      unless column.instance_of? Integer
        raise TypeError, "Expected #{Integer}, got #{colum.class}"
      end
      raise 'Invalid column' unless column.positive?

      @column = column
    end
  end
end
